/* Copyright (c) Sun Microsystems 2006. All rights reserved.
   This file is part of OpenOffice.org project.
   
   Written by Ramón García Fernández <ramon.garcia.f+openoffice@gmail.com>.
   This work was supported by a Google Summer of Code grant.
*/

/* Code for showing output in IDL/UNO language */

#include "browser_lang.hxx"
#include "field_data.hxx"
#include <rtl/ustrbuf.hxx>
#include <com/sun/star/beans/PropertyAttribute.hpp>

namespace objectbrowser_impl {
using namespace ::rtl;
using namespace ::com::sun::star::reflection;

class UnoBrowserLang: public BrowserLang {
    SvLBoxEntry* insertMethod(SvTreeListBox& t, SvLBoxEntry* parent, const Reference<XIdlMethod>& m) const;
    
    SvLBoxEntry* insertProperty(SvTreeListBox& t, SvLBoxEntry* parent, const Property& p) const;

    SvLBoxEntry* insertAttribute(SvTreeListBox& t, SvLBoxEntry* parent, const Reference<XIdlField> & p) const;
};


static const UnoBrowserLang instance;
const BrowserLang* unoLang = &instance;


static OUStringBuffer methodString(const Reference<XIdlMethod>& m)
{
    OUStringBuffer str;
    str.append(m->getReturnType()->getName());
    str.appendAscii(RTL_CONSTASCII_STRINGPARAM(" "));
    str.append(m->getName());
    str.appendAscii(RTL_CONSTASCII_STRINGPARAM("("));
    {
	Sequence<ParamInfo> param_infos = m->getParameterInfos();
	for (unsigned int i = 0; i < (unsigned int) param_infos.getLength(); i++) {
	    if (i > 0) {
		str.appendAscii(RTL_CONSTASCII_STRINGPARAM(", "));
	    }
	    switch(param_infos[i].aMode) {
		case (ParamMode_IN):
		    str.appendAscii("[in] ");
		    break;
		case (ParamMode_OUT):
		    str.appendAscii("[out] ");
		    break;
		case (ParamMode_INOUT):
		    break;
	    }
	    str.append(param_infos[i].aType->getName());
	    str.appendAscii(RTL_CONSTASCII_STRINGPARAM(" "));
	    str.append(param_infos[i].aName);
	}
    }
    str.appendAscii(RTL_CONSTASCII_STRINGPARAM(");"));
    return str;
}

SvLBoxEntry* UnoBrowserLang::insertMethod(SvTreeListBox& t, SvLBoxEntry* parent, const Reference<XIdlMethod>& m) const
{
    OUString s = methodString(m).makeStringAndClear();
    SvLBoxEntry* method_entry = t.InsertEntry(XubString(s), parent);
    return method_entry;
}

SvLBoxEntry* UnoBrowserLang::insertProperty(SvTreeListBox& t, SvLBoxEntry* parent, const Property& p) const
{
    OUStringBuffer str;
    if (p.Attributes != 0) {
	using namespace ::com::sun::star::beans::PropertyAttribute;
	const struct property_attribute {
	    short bit;
	    OUString name;
	} attributes[] = {
	    {MAYBEVOID, OUString(RTL_CONSTASCII_USTRINGPARAM("MAYBEVOID"))},
	    {BOUND, OUString(RTL_CONSTASCII_USTRINGPARAM("BOUND"))},
	    {CONSTRAINED, OUString(RTL_CONSTASCII_USTRINGPARAM("CONSTRAINED"))},
	    {TRANSIENT, OUString(RTL_CONSTASCII_USTRINGPARAM("TRANSIENT"))},
	    {READONLY, OUString(RTL_CONSTASCII_USTRINGPARAM("READONLY"))},
	    {MAYBEAMBIGUOUS, OUString(RTL_CONSTASCII_USTRINGPARAM("MAYBEAMBIGUOUS"))},
	    {MAYBEDEFAULT, OUString(RTL_CONSTASCII_USTRINGPARAM("MAYBEDEFAULT"))},
	    {REMOVEABLE, OUString(RTL_CONSTASCII_USTRINGPARAM("REMOVEABLE"))},
	    {OPTIONAL, OUString(RTL_CONSTASCII_USTRINGPARAM("OPTIONAL"))}};
	str.appendAscii(RTL_CONSTASCII_STRINGPARAM("["));
	bool first = TRUE;
	for (unsigned int i = 0; i < sizeof(attributes)/sizeof(property_attribute); i++) {
	    if ((p.Attributes & attributes[i].bit) != 0) {
		if (!first) {
		    str.appendAscii(RTL_CONSTASCII_STRINGPARAM(" ,"));
		}
		str.append(attributes[i].name);
		first = false;
	    }
	}
	str.appendAscii(RTL_CONSTASCII_STRINGPARAM("] "));
    }
    str.append(p.Type.getTypeName());
    str.appendAscii(RTL_CONSTASCII_STRINGPARAM(" "));
    str.append(p.Name);
    return t.InsertEntry(XubString(str.makeStringAndClear()), parent);
}

SvLBoxEntry* UnoBrowserLang::insertAttribute(SvTreeListBox& t, SvLBoxEntry* parent, const Reference<XIdlField> & p) const
{
    OUStringBuffer str;
    FieldAccessMode m = p->getAccessMode();
    if (m == FieldAccessMode_READONLY) {
	str.appendAscii(RTL_CONSTASCII_STRINGPARAM("readonly "));
    } else if (m == FieldAccessMode_WRITEONLY) {
	str.appendAscii(RTL_CONSTASCII_STRINGPARAM("writeonly "));
    }
    str.append(p->getType()->getName());
    str.appendAscii(RTL_CONSTASCII_STRINGPARAM(" "));
    str.append(p->getName());
    return t.InsertEntry(XubString(str.makeStringAndClear()), parent);
}


};
